package cn.com.ctop.kuaishou.modules.ai.service.impl;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.mapper.UserAllocationMapper;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.LoadFileUtil;
import cn.com.ctop.kuaishou.modules.ai.entity.KuaishouCrowdPack;
import cn.com.ctop.kuaishou.modules.ai.mapper.KuaishouCrowdPackMapper;
import cn.com.ctop.kuaishou.modules.ai.service.IKuaishouCrowdPackService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouInterfaceService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouPopulationService;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * 人群报表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-03-26
 */
@Slf4j
@Service
public class KuaishouCrowdPackServiceImpl extends ServiceImpl<KuaishouCrowdPackMapper, KuaishouCrowdPack> implements IKuaishouCrowdPackService {

    @Autowired
    private KuaishouCrowdPackMapper crowdPackMapper;

    @Autowired
    private ICtopOauthTokenService tokenService;

    @Autowired
    private IKuaishouPopulationService iKuaishouPopulationService;

    @Autowired
    private IKuaishouInterfaceService kuaishouInterfaceService;

    @Autowired
    UserAllocationMapper userAllocationMapper;

    @Value("${zip.local.download-path}")
    private String downloadPath;


    @Override
    public Result<Object> queryPageList(KuaishouCrowdPack kuaishouCrowdPack, Integer pageNo, Integer pageSize) {
        Long total = crowdPackMapper.getTotal(kuaishouCrowdPack.getStatus(), kuaishouCrowdPack.getAccountName(), kuaishouCrowdPack.getOrientationName());
        PageHelper.startPage(pageNo, pageSize, false);
        List<JSONObject> list = crowdPackMapper.queryPageList(kuaishouCrowdPack.getStatus(), kuaishouCrowdPack.getAccountName(), kuaishouCrowdPack.getOrientationName());
        for (JSONObject object : list) {
            Long count = crowdPackMapper.queryProjectCount(object.getString("signature"));
            object.put("projectCount", Check.isNull(count) ? 0 : count);
        }
        PageInfo pageInfo = new PageInfo(list);
        pageInfo.setTotal(total);
        return Result.ok(pageInfo);
    }

    @Override
    public Result<Object> uploadCrowdPack(Integer type, String orientationName, Long accountId, String url) {
        StringBuffer path = new StringBuffer();
        path.append(downloadPath).append(DateUtils.getNowDate(DateUtils.WEB_FORMAT)).append("/").append(UUID.randomUUID().toString()).append("/");
        log.info("-------下载到服务器地址：{}", path.toString());
        if (!url.contains("http")) {
            url = "https:".concat(url);
        }
        String newFileUrl = LoadFileUtil.downLoadFromUrl(url, path.toString());
        String md5 = null;
        try {
            md5 = LoadFileUtil.getMD5(newFileUrl);
        } catch (IOException e) {
            log.error("获取MD5签名失败", e);
        }
        QueryWrapper<KuaishouCrowdPack> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("account_id", accountId);
        queryWrapper.eq("signature", md5);
        queryWrapper.eq("data_status", 1);
        queryWrapper.last("limit 1");
        KuaishouCrowdPack one = crowdPackMapper.selectOne(queryWrapper);
        if (!Check.isNull(one)) {
            return Result.error("本账户已上传过该人群包，人群包名为：" + one.getOrientationName());
        }
        CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
        JSONObject jsonObject = iKuaishouPopulationService.updatePopulation(token, type, orientationName, newFileUrl);
        LoadFileUtil.delFile(newFileUrl);
        if (!Check.isNull(jsonObject)) {
            if (jsonObject.getInteger("code") == 0) {
                KuaishouCrowdPack crowdPack = JSONObject.toJavaObject(jsonObject.getJSONObject("data"), KuaishouCrowdPack.class);
                crowdPack.setSignature(md5);
                crowdPack.setUrl(url);
                crowdPack.setMessage(jsonObject.getString("message"));
                crowdPack.setStatDate(DateUtils.formatDate(new Date()));
                crowdPack.setDataStatus(1);
                this.save(crowdPack);
            } else {
                KuaishouCrowdPack crowdPack = new KuaishouCrowdPack();
                crowdPack.setAccountId(accountId);
                crowdPack.setOrientationName(orientationName);
                crowdPack.setSignature(md5);
                crowdPack.setUrl(url);
                crowdPack.setMessage(jsonObject.getString("message"));
                crowdPack.setStatDate(DateUtils.formatDate(new Date()));
                crowdPack.setDataStatus(0);
                this.save(crowdPack);
                return Result.error(jsonObject.getString("message"));
            }
        }
        return Result.ok("success");
    }

    @Override
    public Result<Object> accountPush(JSONObject request) {
        JSONArray accountIds = request.getJSONArray("accountIds");
        Long orientationId = request.getLong("orientationId");
        Long accountId = request.getLong("accountId");
        CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
        QueryWrapper<KuaishouCrowdPack> wrapper = new QueryWrapper<>();
        wrapper.eq("account_id", accountId);
        wrapper.eq("orientation_id", orientationId);
        wrapper.last("limit 1");
        KuaishouCrowdPack kuaishouCrowdPack = crowdPackMapper.selectOne(wrapper);
        JSONObject jsonObject = iKuaishouPopulationService.pushAccountsPopulation(token, orientationId, accountIds);
        if (jsonObject.getInteger("code") == 0) {
            List<KuaishouCrowdPack> list = new ArrayList<>();
            JSONObject data = jsonObject.getJSONObject("data");
            if (!Check.isNull(data)) {
                JSONArray success = data.getJSONArray("success");
                for (Object account : success) {
                    kuaishouCrowdPack.setAccountId(Long.valueOf(String.valueOf(account)));
                    kuaishouCrowdPack.setMessage("PUSH OK");
                    kuaishouCrowdPack.setStatDate(DateUtils.formatDate(new Date()));
                    list.add(kuaishouCrowdPack);
                }
                JSONArray fail = data.getJSONArray("fail");
                for (Object account : fail) {
                    Long aaccount = Long.valueOf(String.valueOf(account));
                    KuaishouCrowdPack crowdPack = crowdPackMapper.queryByAccountId(aaccount, orientationId);
                    if (Check.isNull(crowdPack)) {
                        kuaishouCrowdPack.setAccountId(aaccount);
                        kuaishouCrowdPack.setMessage("推送失败");
                        kuaishouCrowdPack.setStatDate(DateUtils.formatDate(new Date()));
                        kuaishouCrowdPack.setStatus(null);
                        kuaishouCrowdPack.setType(null);
                        kuaishouCrowdPack.setCoverNum(null);
                        kuaishouCrowdPack.setRecordSize(null);
                        kuaishouCrowdPack.setMatchSize(null);
                        list.add(kuaishouCrowdPack);
                    }
                }
                if (list.size() > 0) {
                    crowdPackMapper.replaceBatch(list);
                }
            }
        } else {
            return Result.error(jsonObject.getString("message"));
        }
        return Result.ok("success");
    }

    @Override
    public void crowdListQuery() {
        List<KuaishouCrowdPack> crowdList = crowdPackMapper.queryListByStatus();
        for (int i = 0; i <= 7; i++) {
            if (i != 2) {
                for (KuaishouCrowdPack crowdPack : crowdList) {
                    CtopOauthToken token = tokenService.getTokenByAccountId(crowdPack.getAccountId());
                    JSONObject jsonObject = iKuaishouPopulationService.getPopulationList(crowdPack.getAccountId(), token.getAccessToken(), i);
                    if (jsonObject.getInteger("code") == 0) {
                        JSONObject data = jsonObject.getJSONObject("data");
                        if (!Check.isNull(data)) {
                            JSONArray details = data.getJSONArray("details");
                            if (details.size() > 0) {
                                JSONArray datas = new JSONArray();
                                for (int j = 0; j < details.size(); j++) {
                                    JSONObject obj = details.getJSONObject(j);
                                    if (Check.isNull(crowdPackMapper.queryByAccountId(obj.getLong("account_id"), obj.getLong("orientation_id")))) {
                                        KuaishouCrowdPack oldPack = crowdPackMapper.queryByAccountId(null, obj.getLong("orientation_id"));
                                        KuaishouCrowdPack crowdPackEntity = JSONObject.toJavaObject(obj, KuaishouCrowdPack.class);
                                        if (!Check.isNull(oldPack)) {
                                            crowdPackEntity.setStatDate(oldPack.getStatDate());
                                            crowdPackEntity.setUrl(oldPack.getUrl());
                                            crowdPackEntity.setSignature(oldPack.getSignature());
                                        }
                                        crowdPackEntity.setMessage("PUSH OK");
                                        crowdPackEntity.setDataStatus(1);
                                        this.save(crowdPackEntity);
                                    } else {
                                        datas.add(obj);
                                    }
                                }
                                if (datas.size() > 0) {
                                    crowdPackMapper.updateBatch(datas);
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    @Override
    public Result<Object> queryRel(String type, String signature, Integer pageNo, Integer pageSize) {
        PageHelper.startPage(pageNo, pageSize);
        List<JSONObject> list = null;
        if ("1".equals(type)) {
            list = crowdPackMapper.queryRelProject(signature);
        } else {
            list = crowdPackMapper.queryRelAccounts(signature);
        }
        PageInfo pageInfo = new PageInfo(list);
        return Result.ok(pageInfo);
    }

    @Override
    public Result<Object> queryFailReason(String signature, Integer pageNo, Integer pageSize) {
        PageHelper.startPage(pageNo, pageSize);
        List<JSONObject> list = crowdPackMapper.queryFailReason(signature);
        PageInfo pageInfo = new PageInfo(list);
        return Result.ok(pageInfo);
    }

    @Override
    public Result<Object> updateDataStatus(KuaishouCrowdPack kuaishouCrowdPack) {
        CtopOauthToken token = tokenService.getTokenByAccountId(kuaishouCrowdPack.getAccountId());
        JSONObject object = iKuaishouPopulationService.deletePopulation(kuaishouCrowdPack.getAccountId(), kuaishouCrowdPack.getOrientationId(), token.getAccessToken());
        if (!Check.isNull(object)) {
            if (object.getInteger("code") == 0) {
                if (this.removeById(kuaishouCrowdPack.getId())) {
                    return Result.ok("删除成功");
                }
            }
            return Result.ok("删除失败，".concat(object.getString("message")));
        }
        return Result.ok("删除失败");
    }
}
